<?php

declare(strict_types=1);

namespace Drupal\search_api_typesense;

use Drupal\ai\AiProviderPluginManager;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\key\KeyRepositoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Adapter for AI providers.
 */
class AiProviderAdapter {

  private array $supportedProviders = ['openai'];

  /**
   * AiProviderAdapter constructor.
   *
   * @param \Drupal\ai\AiProviderPluginManager|null $aiProviderPluginManager
   *   The AI provider plugin manager.
   * @param \Drupal\key\KeyRepositoryInterface $keyRepository
   *   The key repository.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(
    private readonly ?AiProviderPluginManager $aiProviderPluginManager,
    private readonly KeyRepositoryInterface $keyRepository,
    private readonly LoggerInterface $logger,
  ) {}

  /**
   * Check if AI support is available.
   *
   * @return bool
   *   TRUE if AI support is available, FALSE otherwise.
   */
  public function isAiSupportAvailable(): bool {
    return $this->aiProviderPluginManager !== NULL;
  }

  /**
   * Get the available embedding models.
   *
   * @return array
   *   The available embedding models.
   */
  public function getEmbeddingModels(): array {
    try {
      $models = [];
      $providers = $this->aiProviderPluginManager->getProvidersForOperationType(
        'embeddings',
      );

      foreach ($providers as $provider) {
        /** @var \Drupal\ai\AiProviderInterface $provider_loaded */
        $provider_loaded = $this->aiProviderPluginManager->createInstance(
          $provider['id'],
        );

        foreach ($provider_loaded->getConfiguredModels(
          'embeddings',
        ) as $model_id => $model_label) {
          $models[\sprintf('%s/%s', $provider['id'], $model_id)] = \sprintf(
            '%s: %s',
            $provider['id'],
            $model_label,
          );
        }
      }

      return $this->filterSupportedProvider($models);
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());

      return [];
    }
  }

  /**
   * Get the configuration for an embedding model.
   *
   * @param string|null $embedding_model
   *   The embedding model.
   *
   * @return array
   *   The configuration.
   */
  public function getEmbeddingModelConfig(?string $embedding_model): array {
    if ($embedding_model === NULL) {
      return [];
    }

    try {
      [$provider_id, $model_id] = \explode('/', $embedding_model);

      /** @var \Drupal\ai\Base\AiProviderClientBase $provider_loaded */
      $provider_loaded = $this
        ->aiProviderPluginManager
        ->createInstance($provider_id);

      $provider_config = $provider_loaded->getConfig();

      $embedding_config = [
        'model_name' => \sprintf('%s/%s', $provider_id, $model_id),
        'api_key' => $this->getKeyValue($provider_config),
      ];

      $url = $provider_config->get('url');
      if ($url !== NULL) {
        $embedding_config['url'] = $url;
      }

      return $embedding_config;
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());

      return [];
    }
  }

  /**
   * Get the available conversation models.
   *
   * @return array
   *   The available conversation models.
   */
  public function getConversationModels(): array {
    try {
      $models = [];
      $providers = $this->aiProviderPluginManager->getProvidersForOperationType(
        'chat',
      );

      foreach ($providers as $provider) {
        /** @var \Drupal\ai\AiProviderInterface $provider_loaded */
        $provider_loaded = $this->aiProviderPluginManager->createInstance(
          $provider['id'],
        );

        foreach ($provider_loaded->getConfiguredModels(
          'chat',
        ) as $model_id => $model_label) {
          $models[\sprintf('%s/%s', $provider['id'], $model_id)] = \sprintf(
            '%s: %s',
            $provider['id'],
            $model_label,
          );
        }
      }

      return $this->filterSupportedProvider($models);
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());

      return [];
    }
  }

  /**
   * Get the configuration for a conversation model.
   *
   * @param string|null $conversation_model
   *   The conversation model.
   *
   * @return array
   *   The configuration.
   */
  public function getConversationModelConfig(
    ?string $conversation_model,
  ): array {
    if ($conversation_model === NULL) {
      return [];
    }

    try {
      [$provider_id, $model_id] = \explode('/', $conversation_model);

      /** @var \Drupal\ai\Base\AiProviderClientBase $provider_loaded */
      $provider_loaded = $this
        ->aiProviderPluginManager
        ->createInstance($provider_id);

      $provider_config = $provider_loaded->getConfig();

      return [
        'api_key' => $this->getKeyValue($provider_config),
      ];
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());

      return [];
    }
  }

  /**
   * Extract the key value from the configuration.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $config
   *   The configuration.
   *
   * @return string
   *   The key value.
   */
  private function getKeyValue(ImmutableConfig $config): string {
    $key = $this
      ->keyRepository
      ->getKey($config->get('api_key'));

    return $key?->getKeyValue() ?? '';
  }

  /**
   * Filter out providers that are not supported by the adapter.
   *
   * @param array $models
   *   The providers to filter.
   *
   * @return array
   *   The supported providers.
   */
  private function filterSupportedProvider(array $models): array {
    return \array_filter($models, function ($model_id) {
      [$provider_id, $model_name] = \explode('/', $model_id);

      return \in_array($provider_id, $this->supportedProviders, TRUE);
    }, ARRAY_FILTER_USE_KEY);
  }

}
